/* ++++++++++
	file: driver.c

	info: based upon driver.c 'A skeletal device driver'
	         This driver implements the BJL communication protocols with
	         the Atari Juagar BJL developement System by 42Bastian Schick.
	         See readme.txt for more details on Jaguar developement.

	driver author: Robert Jurziga, (moveq@gmx.net)
+++++ */

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// I N C L U D E S
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
#include <SupportDefs.h>
#include <KernelExport.h>
#include <Drivers.h>
#include <Errors.h>
#include <ByteOrder.h>
#include "../driver.h"



// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// D E F I N E S
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
#define	BJL_DEVICE_NAME	"misc/bjl"



// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// P R O T O T Y P E S
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
void	SendLong	 	(uint32 value);
uint32	SendByte 		(uint8 value);




// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// D A T A
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
uint32		pBase;
bigtime_t	waiter = 1;



/* ----------
	init_hardware - called once the first time the driver is loaded
----- */
status_t
init_hardware (void)
{
	return B_OK;
}



/* ----------
	init_driver - optional function - called every time the driver
	is loaded.
----- */
/*
status_t
init_driver (void)
{
	return B_OK;
}
*/

/* ----------
	uninit_driver - optional function - called every time the driver
	is unloaded
----- */
/*
void
uninit_driver (void)
{
}
*/
	

/* ----------
	my_device_open - handle open() calls
----- */
static status_t
my_device_open (const char *name, uint32 flags, void** cookie)
{
	*cookie = NULL;
	return B_OK;
}


/* ----------
	my_device_read - handle read() calls
----- */

static status_t
my_device_read (void* cookie, off_t position, void *buf, size_t* num_bytes)
{
	*num_bytes = 0;				/* tell caller nothing was read */
	return 	(B_OK);

}


/* ----------
	my_device_write - handle write() calls
----- */

static status_t
my_device_write (void* cookie, off_t position, const void* buffer, size_t* num_bytes)
{
	*num_bytes = 0;				/* tell caller nothing was written */
	return (B_OK);
}


/* ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
	my_device_control - handle ioctl calls
----- */

static status_t
my_device_control (void* cookie, uint32 op, void* arg, size_t len)
{

	uint32 	*buf;
	uint32	bufsize;
	int32	cksum = 0;
	int i;
	uint32 data;
	uint32 addr;
	IOCtlData *icd;
	 	
	set_thread_priority ( find_thread(NULL), B_LOW_PRIORITY	);
	
	icd			= (IOCtlData *)arg;		// cast pointer
	pBase 			= icd->Port;			// set the pBase address
	buf			= (uint32 *)icd->buffer;	// buffer to send 
	bufsize			= icd->bufferSize;		// buffer size
	addr			= icd->addr;			// which address
	
	switch	 (op)	{
		case	BJL_SEND:
			SendLong ( 0 );			// syncronize	
			SendLong ( 0x22071970 );		//  MAGIC //
			SendLong ( addr );			// starting address
			SendLong ( bufsize +4 );		// buffer size
			
			for (i=0; i<bufsize; i+=4) {					// send data
				snooze(waiter);
				data =  B_BENDIAN_TO_HOST_INT32(*buf++);	// to Jauguar 64
				cksum -= data;					//
				SendLong  ( data );					//
				}							//
			SendLong ( cksum );						// send CheckSum
						
			return (B_OK);
			break;
			
		default:
			return (B_BAD_VALUE);
		}

	return (B_BAD_VALUE);
}


/* ----------
	my_device_close - handle close() calls
----- */

static status_t
my_device_close (void* cookie)
{
	return B_OK;
}


/* -----
	my_device_free - called after the last device is closed, and after
	all i/o is complete.
----- */
static status_t
my_device_free (void* cookie)
{
	return B_OK;
}


/* -----
	null-terminated array of device names supported by this driver
----- */

static const char *my_device_name[] = {
	BJL_DEVICE_NAME,
	NULL
};

/* -----
	function pointers for the device hooks entry points
----- */

device_hooks my_device_hooks = {
	my_device_open, 			/* -> open entry point */
	my_device_close, 			/* -> close entry point */
	my_device_free,			/* -> free cookie */
	my_device_control, 		/* -> control entry point */
	my_device_read,			/* -> read entry point */
	my_device_write			/* -> write entry point */
};

/* ----------
	publish_devices - return a null-terminated array of devices
	supported by this driver.
----- */

const char**
publish_devices()
{
	return my_device_name;
}

/* ----------
	find_device - return ptr to device hooks structure for a
	given device name
----- */

device_hooks*
find_device(const char* name)
{
	return &my_device_hooks;
}



// ********
// The following low-level routines were written by 42Bastian Schick
//  ******************************************************************
//

void	SendLong (uint32 value)
{
	SendByte (value);
	SendByte (value>>8);
	SendByte (value >>16);
	SendByte (value >> 24);
}





uint32	SendByte (uint8 value)
{
	__asm__("

	movl	pBase, %edx

	incw	%dx
l94a:	inb	%dx,%al
	testb	%al,%al
	jns	l94a
	
	movl	pBase,%edx
	movl	0x8(%ebp),%eax
	outb	%al,%dx
	
	incw	%dx
	incw	%dx
	movb	$1,%al
	outb	%al,%dx

	decw	%dx
l100a:	inb	%dx,%al
	testb	%al,%al
	js	l100a
	
	movl	pBase,%edx
	incw	%dx
	incw	%dx
	
	xorb	%al,%al
//	movb	$0x20,%al
	outb	%al,%dx
	
	");
}
